#include "./Power.h"
#include "../Static.h"
#include <ArduinoJson.h>

void
doBrightness(Args& args, Print& out)
{
  String nextVal = args[1];
  uint8_t newBrightness = (uint8_t)atoi(nextVal.c_str());
  MainLoop::instance()->dispatch(InputEvent{InputEvent::SetBrightness, newBrightness});
}

void
doOn(Args& args, Print& out)
{
  MainLoop::instance()->dispatch(InputEvent{InputEvent::SetPower, 255});
}

void
doOff(Args& args, Print& out)
{
  MainLoop::instance()->dispatch(InputEvent{InputEvent::SetPower, 0});
}

void
doForceBrightness(Args& args, Print& out)
{
  String nextVal = args[1];
  uint8_t newBrightness = (uint8_t)atoi(nextVal.c_str());
  Static<Power>::instance()->forceBrightness(newBrightness);
}

void
Power::forceBrightness(uint8_t v)
{
  m_forced = true;
  FastLED.setBrightness(v);
}

const std::vector<Command> _commands = {
  {"brightness", doBrightness},
  {"brightness-force", doForceBrightness},
  {"on", doOn},
  {"off", doOff}
};

const std::vector<Command>&
Power::commands() const
{
  return _commands;
}

void
Power::handleConfigChange(const Configuration& config)
{
    m_milliamps = config.get("power.milliamps", m_milliamps);
    m_voltage = config.get("power.volts", m_voltage);
    m_useBPM = config.get("power.useBPM", m_useBPM);

    if (m_voltage == 0 || m_milliamps == 0) {
      Log.notice("power: Impossible power config: %dma @ %dv", m_milliamps, m_voltage);
      m_valid = false;
    } else {
      Log.notice("power: Configured to use %dma @ %dv", m_milliamps, m_voltage);
      m_valid = true;
      FastLED.setMaxPowerInVoltsAndMilliamps(m_voltage, m_milliamps);
    }
}


STATIC_ALLOC(Power);
STATIC_TASK(Power);
